"""add TorrentLink.dead

Revision ID: 566cc0cdd00a
Revises: 376061fc33e
Create Date: 2016-05-22 18:53:55.675715

"""

# revision identifiers, used by Alembic.
revision = '566cc0cdd00a'
down_revision = '376061fc33e'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('torrent_link', sa.Column('dead', sa.Boolean(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release_monitor', sa.Column('torrent_cached', sa.BOOLEAN(), nullable=True))
    ### end Alembic commands ###
